from django.utils.text import Truncator
from django.shortcuts import get_object_or_404
from django.conf import settings

from evotecore.mixins import SelectedMenuMixin, PermissionListMixin

from .forms import (MessageAnonymousForm, MessageAuthenticatedForm,
                    MessageAuthenticatedBaseForm)
from .models import Message
from .menus import Menu


class MessageBaseMixin(SelectedMenuMixin):
    """ Defines base settings for messages """
    claim = False
    response = False

    def get_template_names(self):
        base_dir = getattr(settings, "EVOTEMESSAGES_TEMPLATE_BASE_DIR")
        return [base_dir + self.template]


class MessageFormMixin(MessageBaseMixin):
    """ Defines form settings """
    form_class = MessageAnonymousForm

    def get_form_class(self):
        if self.claim and self.request.user.is_authenticated() or\
                self.response:
            return MessageAuthenticatedBaseForm
        elif self.request.user.is_authenticated():
            return MessageAuthenticatedForm
        return self.form_class

    def get_form_kwargs(self):
        kwargs = super(MessageFormMixin, self).get_form_kwargs()
        if self.request.user.is_authenticated():
            kwargs.update({
                'user': self.request.user
            })
        return kwargs

    def get_initial(self):
        initial = super(MessageFormMixin, self).get_initial()
        user = self.request.user
        if 'msg_slug' in self.kwargs:
            parent = get_object_or_404(Message, slug=self.kwargs['msg_slug'])
            initial.update({
                'subject': Truncator('Re: ' + parent.subject).chars(260)
            })
        if user.is_authenticated():
            initial.update({
                'name': user.name,
                'email': user.get_email()
            })
        return initial


class MessageListMixin(PermissionListMixin, MessageBaseMixin):
    """ Defines base settings for lists view """

    model = Message
    paginate_by = 10
    folder = Message.Folder.ARCHIVED
    template = "evotemessages/message_list.html"
    selected_menu = Menu.MESSAGES
    permission_required = "evotemessages.view_message"

    def get_context_data(self, *args, **kwargs):
        context = super(MessageListMixin, self).get_context_data(
            *args, **kwargs)
        context.update({
            'is_inbox': self.folder == Message.Folder.RECEIVED,
            'is_sentbox': self.folder == Message.Folder.SENT,
            'is_archivebox': self.folder == Message.Folder.ARCHIVED,
        })
        return context

    def get_queryset(self):
        raise NotImplementedError()
