from __future__ import unicode_literals

from django.views.generic import ListView, DetailView
from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import login_required
from django.views.generic.base import View
from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import ugettext_lazy as _

from guardian.shortcuts import assign_perm
from evotecore.mixins import PermissionRequiredMixin, FeedbackMessageMixin
from evoteajaxforms.views import FormJsonSimpleView

from .templatetags.messages_tags import nested_url
from .models import Message
from .mixins import MessageFormMixin, MessageBaseMixin, MessageListMixin
from .registers import recipient_register
from .menus import Menu


@method_decorator(login_required, name="dispatch")
class MessagesReceivedView(MessageListMixin, ListView):
    """ """
    folder = Message.Folder.RECEIVED

    def get_queryset(self):
        return Message.objects.inbox(self.request.user)


@method_decorator(login_required, name="dispatch")
class MessagesSentView(MessageListMixin, ListView):
    """ """
    folder = Message.Folder.SENT

    def get_queryset(self):
        return Message.objects.sentbox(self.request.user)


@method_decorator(login_required, name="dispatch")
class MessagesArchivedView(MessageListMixin, ListView):
    """ """
    folder = Message.Folder.ARCHIVED

    def get_queryset(self):
        return Message.objects.archivebox(self.request.user)


@method_decorator(login_required, name="dispatch")
class ArchiveMessageView(FeedbackMessageMixin, PermissionRequiredMixin, View):
    permission_required = "evotemessages.view_message"
    success_message = _("Message was archived successfully")

    def get_permission_object(self):
        msg_pk = self.kwargs.get('pk', None)
        return get_object_or_404(Message, pk=msg_pk)

    def get(self, request, *args, **kwargs):
        slug = kwargs.get('slug', '')
        message = self.get_permission_object()
        message.folder = Message.Folder.ARCHIVED
        message.save()
        self.set_success_message()
        url = nested_url('messages-received', slug=slug)
        return redirect(url)


@method_decorator(login_required, name="dispatch")
class MessageNewView(MessageFormMixin, FormJsonSimpleView):
    """ """
    prefix = 'newmsg'
    template = "evotemessages/message_form.html"
    selected_menu = Menu.MESSAGES
    success_message = _("Your message was sent successfully.")

    def get_success_url(self):
        return nested_url('messages-received')

    def form_valid(self, form):
        instance = form.save(commit=False)
        form_recipients = form.cleaned_data.get('recipients', [])
        recipients = []
        for name in form_recipients:
            recipients += recipient_register.get_all_users(name)
        instance.save()
        instance.recipients = recipients
        if instance.sender:
            assign_perm("view_message", instance.sender, instance)
        Message.objects.send(instance)
        self.set_success_message()
        return self.json_to_response()


@method_decorator(login_required, name="dispatch")
class MessageView(PermissionRequiredMixin, MessageBaseMixin, DetailView):
    """ """
    model = Message
    template = "evotemessages/message_view.html"
    slug_url_kwarg = "msg_slug"
    permission_required = "evotemessages.view_message"
    selected_menu = Menu.MESSAGES

    def get(self, request, *args, **kwargs):
        response = super(MessageView, self).get(request, *args, **kwargs)
        self.object.mark_as_read()
        return response


@method_decorator(login_required, name="dispatch")
class MessageResponseView(PermissionRequiredMixin, MessageFormMixin,
                          FormJsonSimpleView):
    """ """
    prefix = 'remsg'
    response = True
    template = "evotemessages/message_form.html"
    slug_url_kwarg = "msg_slug"
    permission_required = "evotemessages.view_message"
    selected_menu = Menu.MESSAGES
    success_message = _("Your message was sent successfully.")

    def get_success_url(self):
        return nested_url('messages-received')

    def get_permission_object(self):
        return Message.objects.get(slug=self.kwargs[self.slug_url_kwarg])

    def form_valid(self, form):
        instance = form.save(commit=False)
        if 'msg_slug' in self.kwargs:
            parent = self.get_permission_object()
            instance.parent = parent
        instance.save()
        if parent.sender:
            instance.recipients = [parent.sender]
        else:
            instance.anonymous_email = parent.anonymous_email
        if instance.sender:
            assign_perm("view_message", instance.sender, instance)
        Message.objects.send(instance)
        self.set_success_message()
        return self.json_to_response()
