from __future__ import unicode_literals

import json
import logging

from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone
from django.conf import settings
from django.apps import apps

from eVotUM.Cripto import pkiutils, eccblind
from evotevotings.admin.models import SecretEntry
from evotevotings.admin.utils import (get_secret_entry,
                                      get_private_key_password)

logger = logging.getLogger('evote.logger')


def get_election_model():
    return apps.get_model("evoteelections.Election")  # noqa


def get_electionvoter_model():
    return apps.get_model("evotevoters.ElectionVoter")  # noqa


def get_election_data(election_id=None, user_id=None):
    # Get all necessary data from the database
    try:
        Election = get_election_model()
        election_obj = Election.objects.get(pk=election_id)
        election_cert = election_obj.certificate
        pr_components = election_obj.blindsig_pr_dash_components

        election = {}

        vs_cert = get_secret_entry(
            SecretEntry.Identifier.VALIDATOR_CERT
        )
        votesys_private_key = get_secret_entry(
            SecretEntry.Identifier.VOTING_PVT_KEY
        )
        pvtkey_password = get_private_key_password()

        combo_id = get_user_election(election_id, user_id)

        # Set the blind pr components
        election['pr_dash_components'] = pr_components

        # Set the certificates to be sent
        election['election_cert'] = election_cert
        election['vs_cert'] = vs_cert

        # Sign the needed objects using the system private key
        # Attempt to sign the combo_id {user + election_id}sv
        errorCode, signed_combo_id = pkiutils.signObject(combo_id,
                                                         votesys_private_key,
                                                         pvtkey_password)
        if (errorCode is not None):
            raise Exception(
                _("(%d) Could not sign user/election object" % errorCode)
            )

        election['combo_id'] = signed_combo_id

        # Attempt to sign the election_id {election_id}sv
        errorCode, signed_election_id = pkiutils.signObject(
            election_id,
            votesys_private_key,
            pvtkey_password
        )
        if (errorCode is not None):
            raise Exception(
                _("(%d) Could not sign election_id" % errorCode)
            )

        election['election_id'] = signed_election_id

        return election

    except ObjectDoesNotExist:
        logger.critical(_("Forged election ID"))
    except Exception as e:
        logger.critical(format(e))
        return False


def get_user_election(election_id=None, user_id=None):
    return {'election_id': str(election_id),
            'user_id': str(user_id)}


def initialize_blind_signatures():
    (init_component, pr_component) = eccblind.initSigner()
    return (init_component, pr_component)


def get_extra_election_data(election_id=None, user_id=None):
    # Get all necessary data from the database
    try:
        # Check if Election exists (Exception raised)
        Election = get_election_model()
        Election.objects.get(pk=election_id)

        # ElectionVoter to fetch the category
        ElectionVoter = get_electionvoter_model()
        electionvoter_obj = ElectionVoter.objects.get(
            election_id=election_id,
            user_id=user_id
        )

        # Validator Service Certificate
        vs_cert = get_secret_entry(
            SecretEntry.Identifier.VALIDATOR_CERT
        )
        if not vs_cert:
            raise Exception(_("No Validator Service Certificate found"))

        # Voting System Private Key
        votesys_private_key = get_secret_entry(
            SecretEntry.Identifier.VOTING_PVT_KEY
        )
        if not votesys_private_key:
            raise Exception(_("No Voting System Private Key found"))

        # Voting System Private Key password
        pvtkey_password = get_private_key_password()
        if not pvtkey_password:
            raise Exception(_("No Voting System Pvt Key password found"))

        # User/Election combo_id {user + election_id}
        combo_id = get_user_election(election_id, user_id)
        combo_id = json.dumps(combo_id)

        # Sign the needed objects using the system private key
        # Attempt to sign the combo_id {user + election_id}sv
        errorCode, signed_combo_id = pkiutils.signObject(
            combo_id,
            votesys_private_key,
            pvtkey_password
        )

        if errorCode:
            raise Exception(_("Could not sign election+user ID"))

        # Attempt to sign the election_id {election_id}sv
        errorCode, signed_election_id = pkiutils.signObject(
            election_id,
            votesys_private_key,
            pvtkey_password
        )

        if errorCode:
            raise Exception(_("Could not sign election ID"))

        return {
            'combo_id': signed_combo_id,
            'vs_cert': vs_cert,
            'election_id': signed_election_id,
            'category_slug': electionvoter_obj.category.slug
        }

    except Exception as e:
        logger.critical(format(e))
        return False


def calc_archive_date(date):
    days = settings.EVOTE_ELECTIONS.get('archive_after')
    return date + timezone.timedelta(days=days)
