from __future__ import unicode_literals

from mock import patch

from django.utils import timezone
from django.test import SimpleTestCase

from evotevotings.processes.models import (Action, ElectoralProcess,
                                           ElectoralAction, ElectoralMember,
                                           ElectoralDocument)
from evoteusers.models import User


class ActionTest(SimpleTestCase):
    """ """

    def test_str_equal_to_designation(self):
        instance = Action(designation="designation")
        self.assertEqual(str(instance), instance.designation)


class ElectoralProcessTest(SimpleTestCase):
    """ """

    def test_str_equal_to_identifier(self):
        """ """
        instance = ElectoralProcess(identifier="identifier")
        self.assertEqual(str(instance), instance.identifier)

    @patch.object(ElectoralProcess, 'electoral_members')
    def test_can_submit_condition(self, electoral_members):
        """ """
        electoral_members.exists.return_value = True
        process = ElectoralProcess()
        self.assertTrue(process.can_submit())
        electoral_members.exists.return_value = False
        self.assertFalse(process.can_submit())
        self.assertEqual(electoral_members.exists.call_count, 2)

    def test_can_publish_condition(self):
        """ """
        published_process = ElectoralProcess(
            publication_date=timezone.now() - timezone.timedelta(minutes=60))
        unpublished_process = ElectoralProcess(
            publication_date=timezone.now() + timezone.timedelta(minutes=60))
        self.assertTrue(published_process.can_publish())
        self.assertFalse(unpublished_process.can_publish())

    @patch.object(ElectoralProcess, 'elections')
    def test_can_close_condition(self, elections):
        """ """
        elections.closed().count.return_value = 2
        elections.count.return_value = 2
        process = ElectoralProcess()
        self.assertTrue(process.can_close())
        elections.count.return_value = 4
        self.assertFalse(process.can_close())
        self.assertEqual(elections.closed().count.call_count, 2)
        self.assertEqual(elections.count.call_count, 2)

    @patch.object(ElectoralProcess, 'elections')
    def test_can_archive_condition(self, elections):
        """ """
        elections.archived().count.return_value = 2
        elections.count.return_value = 2
        process = ElectoralProcess()
        self.assertTrue(process.can_archive())
        elections.count.return_value = 4
        self.assertFalse(process.can_archive())
        self.assertEqual(elections.archived().count.call_count, 2)
        self.assertEqual(elections.count.call_count, 2)

    @patch.object(ElectoralProcess, 'electoral_members')
    @patch('evotevotings.processes.models.ElectoralMember')
    def test_submit_new_transition(self, member_model, electoral_members):
        """ """
        electoral_members.filter().exists.return_value = True
        electoral_members.filter().first.return_value = "test"
        process = ElectoralProcess(state=ElectoralProcess.State.DRAFT)
        process.submit_new()
        self.assertEqual(process.state, ElectoralProcess.State.NEW)
        electoral_members.filter.assert_called_with(
            role=member_model.Role.CHAIRMAN)  # chairman
        electoral_members.filter().exists.assert_called()
        electoral_members.filter().first.assert_called()
        member_model.objects.mark_as_responsible.assert_called_with(
            process, 'test')

    @patch.object(ElectoralProcess, 'elections')
    def test_publish_transition(self, elections):
        """ """
        process = ElectoralProcess(state=ElectoralProcess.State.NEW,
                                   publication_date=timezone.now())
        process.publish()
        self.assertEqual(process.state, ElectoralProcess.State.OPEN)
        elections.all.assert_called()

    def test_close_transition(self):
        """ """
        process = ElectoralProcess(state=ElectoralProcess.State.OPEN)
        process.close()
        self.assertIsNotNone(process.closed_on)
        self.assertEqual(process.state, ElectoralProcess.State.CLOSED)

    def test_archive_transition(self):
        """ """
        process = ElectoralProcess(state=ElectoralProcess.State.CLOSED)
        process.archive()
        self.assertEqual(process.state, ElectoralProcess.State.ARCHIVED)

    def test_can_edit_members_with_state_draft(self):
        process = ElectoralProcess(state=ElectoralProcess.State.DRAFT)
        self.assertTrue(process.can_edit_members)

    @patch.object(ElectoralProcess, 'elections')
    def test_can_edit_members_with_state_not_draft(self, elections):
        elections.filter().exists.return_value = False
        process = ElectoralProcess(state=ElectoralProcess.State.OPEN)
        self.assertTrue(process.can_edit_members)
        elections.filter().exists.return_value = True
        self.assertFalse(process.can_edit_members)
        elections.filter.assert_called()
        elections.filter().exists.assert_called()


class ElectoralActionTest(SimpleTestCase):
    """ """

    def test_str_equal_to_action_and_strength(self):
        electoral_action = ElectoralAction(
            action=Action(designation="designation"),
            strength=ElectoralAction.Strength.SIMPLE)
        text = "{} ({})".format(
            electoral_action.action, electoral_action.get_strength_display())
        self.assertEqual(str(electoral_action), text)


class ElectoralMemberTest(SimpleTestCase):
    """ """

    def test_has_private_key(self):
        instance = ElectoralMember()
        self.assertFalse(instance.has_private_key)
        instance.private_key = b"Dummy content"
        self.assertTrue(instance.has_private_key)

    def test_str_equal_to_designation(self):
        user = User(name="Dev")
        instance = ElectoralMember(user=user, role=0)
        expected = instance.user.name + \
            " (" + instance.get_role_display() + ")"
        self.assertEqual(str(instance), expected)


class ElectoralDocumentTest(SimpleTestCase):
    """ """

    def test_str_equal_to_designation(self):
        instance = ElectoralDocument(designation="designation")
        self.assertEqual(str(instance), instance.designation)
