from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
from django.db.models import Q

from evotemessages.decorators import recipients
from evotemessages.recipients import BaseRecipient

from .models import ElectoralRoll, ElectionVoter


class ElectoralRollBaseRecipient(BaseRecipient):
    """ """

    def get_users(self, name):
        obj = self.get_object(name)
        if obj:
            users = obj.election_voters.values_list('user_id', flat=True)
            return get_user_model().objects.filter(id__in=users)
        return []

    def get_recipient_label(self, obj):
        return self.label_prefix + " " + obj.election.identifier


@recipients
class ElectoralRollRecipient(ElectoralRollBaseRecipient):
    """
    Group of voters of election
    """
    prefix = 'electoral_voter'
    model = ElectoralRoll
    label_prefix = _('Electoral roll of election')

    def get_recipients(self, *args, **kwargs):
        value = kwargs.get('q', '')
        slug = kwargs.get('slug', None)
        electoral_roll = self.get_queryset().get_published_of_process()
        if electoral_roll:
            if slug:
                return electoral_roll.election_voters.filter(
                    election__identifier__icontains=value,
                    election__electoral_process__slug__in=slug)
            else:
                return electoral_roll.election_voters.filter(
                    election__identifier__icontains=value)
        return []

    def check_permission(self, user):
        if hasattr(user, 'is_electoral_member'):
            return user.is_electoral_member
        return False


@recipients
class ElectoralRollMemberRecipient(ElectoralRollBaseRecipient):
    """
    Voter of electoral roll
    """
    prefix = 'electoral_voter_member'
    model = ElectoralRoll
    label_prefix = _('Electoral roll member') + ":"

    def get_recipients(self, *args, **kwargs):
        value = kwargs.get('q', '')
        slug = kwargs.get('slug', None)
        electoral_roll = self.get_queryset().get_published_of_process()
        if electoral_roll:
            if slug:
                return electoral_roll.election_voters.filter(
                    user__name=value,
                    election__electoral_process__slug__in=slug)
            else:
                return electoral_roll.election_voters.filter(user__name=value)
        return []

    def check_permission(self, user):
        if hasattr(user, 'is_electoral_member'):
            return user.is_electoral_member
        return False


@recipients
class ElectoralRollAllRecipient(ElectoralRollBaseRecipient):
    """
    Group of all members of electoral rolls
    """
    prefix = 'electoral_voter_all'
    model = ElectoralRoll
    label_prefix = _('Electoral roll members of all elections of process')

    def get_recipients(self, *args, **kwargs):
        value = kwargs.get('q', '')
        slug = kwargs.get('slug', None)
        electoral_roll = self.get_queryset().get_published_of_process()
        if electoral_roll:
            if slug:
                return electoral_roll.election_voters.filter(
                    election__electoral_process__slug__in=slug)
            else:
                return electoral_roll.election_voters.filter(
                    election__identifier__icontains=value)
        return []

    def check_permission(self, user):
        if hasattr(user, 'is_electoral_member'):
            return user.is_electoral_member
        return False


@recipients
class ElectionVoterRecipient(BaseRecipient):
    """ """
    prefix = 'electoral_voter'
    model = ElectionVoter
    label_prefix = _('Election voters') + ":"

    def get_recipients(self, *args, **kwargs):
        value = kwargs.get('q', '')
        slug = kwargs.get('slug', None)
        if slug:
            return self.get_queryset().filter(
                Q(user__name__icontains=value) |
                Q(user__email__icontains=value),
                election__electoral_process__slug__in=slug)
        else:
            return self.get_queryset().filter(
                Q(user__name__icontains=value) |
                Q(user__email__icontains=value))

    def get_users(self, name):
        obj = self.get_object(name)
        if obj:
            return [obj.user]
        return []

    def get_recipient_label(self, obj):
        return self.label_prefix + " " + obj.user.name

    def check_permission(self, user):
        if hasattr(user, 'is_electoral_member'):
            return user.is_electoral_member
        return False
